<?php


namespace FiloBlu\Referrals\Model;


use FiloBlu\Newsletter\Model\SubscriberInfoFactory;
use FiloBlu\Referrals\Helper\Core;
use Magento\Framework\App\ObjectManager;
use Magento\Invitation\Model\Invitation\Status;
use Magento\Newsletter\Model\SubscriberFactory;

class Invitation extends \Magento\Invitation\Model\Invitation
{
    const XML_PATH_REFERRAL_EMAIL_TEMPLATE = 'filoblu_referrals/general/templateinvitation';


    /**
     * Send invitation email
     *
     * @return bool
     */
    public function sendInvitationEmail()
    {
        $this->makeSureCanBeSent();
        $store = $this->_storeManager->getStore($this->getStoreId());

        $templateIdentifier = $this->_scopeConfig->getValue(
            self::XML_PATH_REFERRAL_EMAIL_TEMPLATE,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $store
        );
        $from = $this->_scopeConfig->getValue(
            self::XML_PATH_EMAIL_IDENTITY,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $store
        );

        $urls = $this->_invitationData->getInvitationUrl($this);

        $templateVars = [
            'message' => $this->getMessage(),
            'store' => $store,
            'store_name' => $store->getGroup()->getName(),
            'inviter_name' => $this->getInviter() ? $this->getInviter()->getName() : null,
        ];

        foreach ($urls as $name => $url) {
            $templateVars[$name] = $url;
        }

        $this->_transportBuilder->setTemplateIdentifier(
            $templateIdentifier
        )->setTemplateOptions(
            ['area' => \Magento\Framework\App\Area::AREA_FRONTEND, 'store' => $this->getStoreId()]
        )->setTemplateVars(
            $templateVars
        )->setFrom(
            $from
        )->addTo(
            $this->getEmail()
        );
        $transport = $this->_transportBuilder->getTransport();
        try {
            $transport->sendMessage();
        } catch (\Magento\Framework\Exception\MailException $e) {
            return false;
        }
        $this->setStatus(Status::STATUS_SENT)->setUpdateDate(true)->save();
        return true;
    }

    public function accept($websiteId , $referralId)
    {
        $coreHelper = $this->getCoreHelper();

        $subscriberFactory = $this->getSubscriberFactory();
        $subscriberInfoFactory = $this->getSubscriberInfoFactory();

        $couponInvitationType = $coreHelper->getSysConfigValue($coreHelper::COUPON_FIELD_INVITATION_TYPE);

        $this->makeSureCanBeAccepted($websiteId);

        $this->getResource()->beginTransaction();
        try {
            $this->setReferralId(
                $referralId
            )->setStatus(
                Status::STATUS_ACCEPTED
            )->setSignupDate(
                $this->dateTime->formatDate(time())
            )->save();

            $inviterId = $this->getCustomerId();

            if ($couponInvitationType === 'newsletter') {
                $email = $this->getEmail();
                $subscriber = $subscriberFactory->create()->loadByEmail($email);
                if (!$subscriber->isSubscribed()) {
                    $subscriberFactory->create()->setData('is_referral', true)->setImportMode(true)->subscribe($email);
                    $subscriber = $subscriberFactory->create()->loadByEmail($email);

                    $subscriberInfoSourceType = $coreHelper->getSysConfigValue($coreHelper::NEWSLETTER_SUBSCRIPTION_SOURCE) ?? 'NO';
                    
                    $subscriberInfo = $subscriberInfoFactory->create()->load($subscriber->getId());
                    $subscriberInfo->setSource($subscriberInfoSourceType);
                    $subscriberInfo->save();
                    
                }
                $this->setData('subscriber_id', $subscriber->getSubscriberId());
            } else {
                $this->assignInvitationDataToCustomer($referralId);
            }


            if ($inviterId) {
                if ($inviterId === $referralId && $this->hasData('subscriber_id')) {
                    $this->getResource()->trackNlSubscriptionReferral($inviterId, $this->getSubscriberId());
                } else {
                    $this->getResource()->trackReferral($inviterId, $referralId);
                }
            }

            $this->getResource()->commit();
        } catch (\Exception $e) {
            $this->getResource()->rollBack();
            throw $e;
        }

        return $this;
    }

    /**
     * Check whether this invitation can be accepted
     *
     * @param int|string $websiteId
     * @return void
     * @throws \Magento\Framework\Exception\InputException
     */
    public function makeSureCanBeAccepted($websiteId = null)
    {
        $coreHelper = $this->getCoreHelper();
        $messageInvalid = __('This invitation is not valid.');
        if (!$this->getId()) {
            $coreHelper->writeLog("Error in newsletter subscribe: no invitation id");
            throw new \Magento\Framework\Exception\InputException($messageInvalid);
        }
        if (!in_array($this->getStatus(), $this->status->getCanBeAcceptedStatuses())) {
            $coreHelper->writeLog("Error in newsletter subscribe: no status");
            throw new \Magento\Framework\Exception\InputException($messageInvalid);
        }
        if (null === $websiteId) {
            $websiteId = $this->_storeManager->getWebsite()->getId();
        }
        if ($websiteId != $this->_storeManager->getStore($this->getStoreId())->getWebsiteId()) {
            $coreHelper->writeLog("Error in newsletter subscribe: wrong website id");
            throw new \Magento\Framework\Exception\InputException($messageInvalid);
        }
    }

    /**
     * @return Core|mixed
     */
    public function getCoreHelper() {
        return ObjectManager::getInstance()->get(Core::class);
    }

    /**
     * @return SubscriberFactory|mixed
     */
    public function getSubscriberFactory() {
        return ObjectManager::getInstance()->get(SubscriberFactory::class);

    }

    /**
     * @return SubscriberInfoFactory|mixed
     */
    public function getSubscriberInfoFactory() {
        return ObjectManager::getInstance()->get(SubscriberInfoFactory::class);
    }
}